package ch.unige.solidify.auth;

import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import ch.unige.solidify.auth.util.OidcTokenExpiredException;
import ch.unige.solidify.model.sso.SsoUser;

@Component
public class SsoAuthenticationFilter extends OncePerRequestFilter {
  private static final Logger log = LoggerFactory.getLogger(SsoAuthenticationFilter.class);

  private UserFactory userFactory;

  @Autowired
  public SsoAuthenticationFilter(UserFactory userFactory) {
    this.userFactory = userFactory;
  }

  @Override
  protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response,
          FilterChain filterChain)
          throws ServletException, IOException {
    if (log.isDebugEnabled()) {
      log.debug("Request " + request.getRequestURI() + " enter SSoAuthenticationFilter");
    }

    Map<String, String> headers = new HashMap<>();
    Enumeration<String> names = request.getHeaderNames();
    while (names.hasMoreElements()) {
      String name = names.nextElement().toLowerCase();
      String value = request.getHeader(name);
      headers.putIfAbsent(name, value);
    }

    try {
      SsoUser ssoUser = userFactory.get(headers);
      if (ssoUser != null && !ssoUser.isEmpty()) {
        Authentication auth = new SsoAuthentication(new SsoUserDetails(ssoUser));
        SecurityContextHolder.getContext().setAuthentication(auth);
        if (log.isDebugEnabled()) {
          log.debug("Request " + request.getRequestURI() + " set securityContext with new generated user details " + ssoUser.toString());
        }
      }
      filterChain.doFilter(request, response);
      if (log.isDebugEnabled()) {
        log.debug("Request " + request.getRequestURI() + " exit SSoAuthenticationFilter");
      }

    } catch (OidcTokenExpiredException e) {
      if (log.isDebugEnabled()) {
        log.debug("Request " + request.getRequestURI() + " OidcTokenExpiredException");
      }
      response.setStatus(498);
      response.setContentType("application/json");
      JSONObject errorJson = new JSONObject(e);
      response.getWriter().print(errorJson);
    }
  }
}
