package ch.unige.solidify.auth;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.model.sso.SsoUser;

public class SsoUserDetails extends SsoUser implements UserDetails {
  private static final long serialVersionUID = 2803592576870554475L;

  // from Spring Security, spring adds the prefix
  @JsonIgnore
  private String rolePrefix = "ROLE_";

  @JsonIgnore
  private Collection<GrantedAuthority> authorities = new ArrayList<>();

  public SsoUserDetails() {
    super();
  }

  public SsoUserDetails(SsoUser ssoUser) {
    for (String role : ssoUser.getRoles()) {
      authorities.add(new SimpleGrantedAuthority(rolePrefix + role));
    }
    super.uniqueId = ssoUser.getUniqueId();
    super.systemId = ssoUser.getSystemId();
    super.cnindividu = ssoUser.getCnindividu();
    super.firstName = ssoUser.getFirstName();
    super.lastName = ssoUser.getLastName();
    super.name = ssoUser.getName();
    super.email = ssoUser.getEmail();
    super.homeorganization = ssoUser.getHomeOrganization();
    super.employeeType = Collections.unmodifiableSet(ssoUser.getEmployeeType());
    super.unigeChStudentCategory = Collections.unmodifiableSet(ssoUser.getUnigeChStudentCategory());
    super.ismemberOf = Collections.unmodifiableSet(ssoUser.getIsmemberOf());
    super.roles = Collections.unmodifiableSet(ssoUser.getRoles());
    super.homeorganizationtype = ssoUser.getHomeorganizationtype();
    super.affiliation = ssoUser.getAffiliation();
    super.unigechoucode = ssoUser.getUnigechoucode();
    super.unigechstudentoucode = ssoUser.getUnigechstudentoucode();
    super.unigechemployeeoucode = ssoUser.getUnigechemployeeoucode();
    super.orgunitdn = ssoUser.getOrgunitdn();
    super.matriculationnumber = ssoUser.getMatriculationnumber();
    super.allProperties = ssoUser.getAllProperties();
    super.oauthToken = ssoUser.getAccessToken();
    super.oauthTokenExpiresTime = ssoUser.getAccessTokenExpiresTime();
    super.userContext = ssoUser.getUserContext();
  }

  @Override
  @JsonIgnore
  public Collection<? extends GrantedAuthority> getAuthorities() {
    return authorities;
  }

  @Override
  @JsonIgnore
  public String getPassword() {
    throw new IllegalStateException("password");
  }

  @Override
  @JsonIgnore
  public String getUsername() {
    return super.getUniqueId();
  }

  @Override
  @JsonIgnore
  public boolean isAccountNonExpired() {
    return true;
  }

  @Override
  @JsonIgnore
  public boolean isAccountNonLocked() {
    return true;
  }

  @Override
  @JsonIgnore
  public boolean isCredentialsNonExpired() {
    return true;
  }

  @Override
  @JsonIgnore
  public boolean isEnabled() {
    return true;
  }

  @Override
  @JsonIgnore
  public boolean isEmpty() {
    return super.getUniqueId() == null;
  }

}
