package ch.unige.solidify.auth;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import ch.unige.solidify.auth.oidc.OidcUserFactory;
import ch.unige.solidify.auth.shibboleth.ShibbolethUserFactory;
import ch.unige.solidify.auth.util.OidcTokenExpiredException;
import ch.unige.solidify.model.sso.SsoUser;

@Component
class UserFactory {

  private static final Logger log = LoggerFactory.getLogger(UserFactory.class);

  @Autowired
  private ShibbolethUserFactory shibbolethUserFactory;

  @Autowired
  private OidcUserFactory oidcUserFactory;

  SsoUser get(Map<String, String> headers) throws OidcTokenExpiredException {
    if (log.isDebugEnabled()) {
      log.debug("SsoUser.get() with headers " + headers);
    }
    if (headers.containsKey(OidcUserFactory.OIDC_ACCESS_TOKEN)) {
      if (headers.containsKey(OidcUserFactory.OIDC_ACCESS_TOKEN_EXPIRES)) {
        return oidcUserFactory.get(headers.get(OidcUserFactory.OIDC_ACCESS_TOKEN),
                new Long(headers.get(OidcUserFactory.OIDC_ACCESS_TOKEN_EXPIRES)), headers);
      } else {
        return oidcUserFactory.get(headers.get(OidcUserFactory.OIDC_ACCESS_TOKEN), -1, headers);
      }
    } else {
      return shibbolethUserFactory.get(headers);
    }
  }
}
