package ch.unige.solidify.auth.oidc;

import java.io.Serializable;
import java.util.Map;
import java.util.Set;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonProperty;

import ch.unige.solidify.auth.util.SsoUserData;
import ch.unige.solidify.model.sso.SsoUser;
import ch.unige.solidify.util.StringTool;

/**
 * Transitional class create via the endpoint userinfo of OIDC and then use this class to create the
 * user of the application
 * <p>
 */

public class OIDCUser extends SsoUser implements Serializable {
  private static final long serialVersionUID = 4957365930002948021L;

  public static SsoUserData toUserData(OIDCUser user) {
    if (user == null) {
      return null;
    }
    SsoUserData result = new SsoUserData();
    result.setEmail(user.getEmail());
    result.setFirstname(user.getFirstName());
    result.setLastname(user.getLastName());
    result.setUniqueid(user.getUniqueId());
    result.setSystemId(user.getSystemId());
    result.setName(user.getName());
    result.setOauthToken(user.getAccessToken());
    result.setOauthTokenExpiresTime(user.getAccessTokenExpiresTime());
    result.setCnindividu(user.getCnindividu());
    result.setIsmemberOf(user.getIsmemberOf());
    result.setRoles(user.getIsmemberOf());
    result.setAllProperties(user.getAllProperties());
    result.setUserContext(user.getUserContext());
    return result;
  }

  @JsonProperty(value = "email")
  @Override
  public String getEmail() {
    return super.getEmail();
  }

  @JsonProperty(value = "given_name")
  @Override
  public String getFirstName() {
    return super.getFirstName();
  }

  @JsonProperty(value = "family_name")
  @Override
  public String getLastName() {
    return super.getLastName();
  }

  @JsonProperty(value = "name")
  @Override
  public String getName() {
    return super.getName();
  }

  @JsonProperty(value = "sub")
  @Override
  public String getUniqueId() {
    return super.getUniqueId();
  }

  public void setUniqueId(String uniqueid) {
    super.uniqueId = uniqueid;
  }

  @JsonProperty(value = "isMemberOf")
  @Override
  @JsonFormat(with = JsonFormat.Feature.ACCEPT_SINGLE_VALUE_AS_ARRAY)
  public Set<String> getIsmemberOf() {
    return super.getIsmemberOf();
  }

  @Override
  public String getCnindividu() {
    String uid = getUniqueId();
    if (!StringTool.isNullOrEmpty(uid) && uid.contains("@")) {
      String[] words = uid.split("@");
      return words[0];
    } else {
      return null;
    }
  }

  public void setOauthToken(String oauthToken) {
    super.oauthToken = oauthToken;
  }

  public void setOauthTokenExpiresTime(long oauthTokenExpiresTime) {
    super.oauthTokenExpiresTime = oauthTokenExpiresTime;
  }

  public void setAllProperties(Map<String, String> allProperties) {
    super.allProperties = allProperties;
  }

  public void setUserContext(String userContext) {
    super.userContext = userContext;
  }

  @Override
  public String toString() {
    return "OIDCUser{" + "oauthToken='" + oauthToken + '\'' + ", oauthTokenExpiresTime='"
            + ", userContext='" + userContext + '\'' + +oauthTokenExpiresTime + '\''
            + ", firstName='" + firstName + '\'' + ", lastName='" + lastName + '\'' + ", email='"
            + email + '\'' + ", homeorganization='" + homeorganization + '\''
            + ", homeorganizationtype='" + homeorganizationtype + '\'' + ", affiliation="
            + affiliation + ", ismemberOf=" + ismemberOf + ", employeeType=" + employeeType
            + ", unigechoucode='" + unigechoucode + '\'' + ", unigechstudentoucode='"
            + unigechstudentoucode + '\'' + ", unigechemployeeoucode='" + unigechemployeeoucode
            + '\'' + ", uniqueId='" + uniqueId + '\'' + ", unigeChStudentCategory="
            + unigeChStudentCategory + ", orgunitdn='" + orgunitdn + '\''
            + ", matriculationnumber='" + matriculationnumber + '\'' + ", cnindividu='" + cnindividu
            + '\'' + ", name='" + name + '\'' + ", roles=" + roles + ", systemId=" + systemId
            + ", allProperties=" + allProperties + '}';
  }
}
