package ch.unige.solidify.auth.oidc;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.HttpServerErrorException;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.client.UnknownHttpStatusCodeException;

import ch.unige.solidify.auth.config.SsoAppConfig;
import ch.unige.solidify.auth.util.OIDCUtil;

@Service
public class OidcUserInfoService {
  private static final Logger log = LoggerFactory.getLogger(OidcUserInfoService.class);

  @Autowired
  Environment environment;
  private SsoAppConfig ssoAppConfig;

  @Autowired
  public OidcUserInfoService(SsoAppConfig ssoAppConfig) {
    this.ssoAppConfig = ssoAppConfig;
  }

  public OIDCUser getOidcUser(String token, long tokenExpiresTime) {
    if (log.isDebugEnabled()) {
      log.debug("OidcUserInfoService.getOidcUser(" + token + ", " + tokenExpiresTime + ")");
    }
    String userInfoLink = OIDCUtil.getOidcServerUrl(environment, ssoAppConfig);
    HttpHeaders headers = new HttpHeaders();
    headers.add("Authorization", "Bearer " + token);
    if (log.isDebugEnabled()) {
      log.debug("Added Header : Authorization " + "Bearer " + token);
    }
    HttpEntity<String> entity = new HttpEntity<>(null, headers);
    RestTemplate restTemplate = new RestTemplate();
    OIDCUser result = null;
    try {
      if (log.isDebugEnabled()) {
        log.debug("Attempt to get OIDCUser from " + userInfoLink);
      }
      result = restTemplate.postForObject(userInfoLink, entity, OIDCUser.class);
    } catch (HttpClientErrorException e) {
      log.error("Exception when invoking url  " + userInfoLink, e);
      throw e;
    } catch (HttpServerErrorException e) {
      log.error("Exception when invoking url  " + userInfoLink, e);
      throw e;
    } catch (UnknownHttpStatusCodeException e) {
      log.error("Exception when invoking url  " + userInfoLink, e);
      throw e;
    } catch (Exception e) {
      log.error("Exception when invoking url  " + userInfoLink, e);
      String resultString = restTemplate.postForObject(userInfoLink, entity, String.class);
      log.error("Resultat invoking url " + resultString, e);
      throw new RuntimeException(
              "Response body  from userInfoLink :" + resultString + " ; Error message : " + e
                      .getMessage() + " CAUSE: " + e.getCause().getMessage(),
              e);

    }
    result.setOauthToken(token);
    result.setOauthTokenExpiresTime(tokenExpiresTime);
    if (log.isDebugEnabled()) {
      log.debug("Get result " + result);
    }
    return result;
  }
}
