package ch.unige.solidify.auth.util;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.test.context.support.WithSecurityContextFactory;

import ch.unige.solidify.auth.SsoAuthentication;
import ch.unige.solidify.auth.SsoUserDetails;
import ch.unige.solidify.model.sso.SsoUser;

public class MockSsoUserSecurityContextFactory
        implements WithSecurityContextFactory<WithSsoMockUser> {

  private static final Logger log =
          LoggerFactory.getLogger(MockSsoUserSecurityContextFactory.class);

  @Override
  public SecurityContext createSecurityContext(WithSsoMockUser unigeShibboMockUser) {
    SecurityContext context = SecurityContextHolder.createEmptyContext();
    SsoUser ssoUser = getUser(unigeShibboMockUser);
    SsoUserDetails principal = new SsoUserDetails(ssoUser);
    Authentication auth = new SsoAuthentication(principal);
    context.setAuthentication(auth);
    return context;
  }

  private SsoUser getUser(WithSsoMockUser mockUser) {
    if (log.isDebugEnabled()) {
      log.debug("MockSsoUserSecurityContextFactory.getUser(" + mockUser + ")");
    }
    SsoUserData userData = new SsoUserData();
    userData.setSystemId(mockUser.systemId());
    userData.setUniqueid(mockUser.uniqueid());
    userData.setUserContext(mockUser.userContext());

    userData.setFirstname(mockUser.firstName());
    userData.setLastname(mockUser.lastname());
    userData.setName(mockUser.name());
    userData.setEmail(mockUser.email());
    userData.setHomeorganization(mockUser.organization());
    userData.setEmployeeType(string2Set(mockUser.employeeType()));

    userData.setUnigeChStudentCategory(string2Set(mockUser.unigeChStudentCategory()));
    userData.setIsmemberOf(string2Set(mockUser.ismemberOf()));
    userData.setRoles(string2Set(mockUser.roles()));

    userData.setHomeorganizationtype(mockUser.homeorganizationtype());
    userData.setAffiliation(string2Set(mockUser.affiliation()));
    userData.setUnigechoucode(mockUser.unigechoucode());
    userData.setUnigechstudentoucode(mockUser.unigechstudentoucode());
    userData.setUnigechemployeeoucode(mockUser.unigechemployeeoucode());
    userData.setOrgunitdn(mockUser.orgunitdn());
    userData.setMatriculationnumber(mockUser.matriculationnumber());
    userData.setCnindividu(mockUser.cnindividu());
    userData.setOauthTokenExpiresTime(mockUser.oauthTokenExpiresTime());
    userData.setOauthToken(mockUser.oauthToken());
    userData
            .setAllProperties(array2Map(mockUser.allPropertiesName(), mockUser.allPropertiesValue()));
    return userData;
  }

  private Map<String, String> array2Map(String[] names, String[] values) {
    Map<String, String> properties = new HashMap<>();
    if (names != null && names.length != 0 && values != null && values.length != 0) {
      int index = 0;
      for (String name : names) {
        properties.put(name, values[index]);
        index++;
      }
    }
    return properties;

  }

  private Set<String> string2Set(String[] input) {
    return new HashSet<>(Arrays.asList(input));
  }
}
