package ch.unige.solidify.auth.util;

import java.util.Arrays;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.env.Environment;

import ch.unige.solidify.auth.config.SsoAppConfig;
import ch.unige.solidify.util.UserTool;

public class OIDCUtil {
  private static final Logger log = LoggerFactory.getLogger(OIDCUtil.class);

  private static final int THIRTY_SECONDS = 30000;
  public static final String ODIC_BASE_LINK_DEV = "http://127.0.0.1:8079/userinfo";
  public static final String ODIC_BASE_LINK_TEST = "https://openidp-test.unige.ch/oxauth/restv1/userinfo";
  public static final String ODIC_BASE_LINK_PROD = "https://openidp.unige.ch/oxauth/restv1/userinfo";

  public static String getOidcServerUrl(Environment environment,
          SsoAppConfig ssoAppConfig) {
    String userInfoLink = ODIC_BASE_LINK_TEST;
    if (log.isDebugEnabled()) {
      log.debug("Active profiles " + Arrays.toString(environment.getActiveProfiles()));
    }
    if (AuthUtility.isProfilesActive(environment, "prd")) {
      return ODIC_BASE_LINK_PROD ;
    }
    if (AuthUtility.isProfilesActive(environment, "dev", "qa")) {
      String appDefinedOidcServerUrl = ssoAppConfig.getOidcServerUrl();
      if (appDefinedOidcServerUrl != null && !appDefinedOidcServerUrl.isEmpty()) {
        return appDefinedOidcServerUrl;
      } else {
        return ODIC_BASE_LINK_DEV ;
      }
    }
    return userInfoLink;
  }

  /**
   * Check if a token need to be refreshed. if access token will be expired in 30 seconds, token needs
   * refreshing
   *
   * @param tokenExpiresTime access token expiring time
   * @return boolean indicate if we need refresh token or not
   */
  public static boolean isTokenNeedFresh(long tokenExpiresTime) {
    long difference = System.currentTimeMillis() - tokenExpiresTime * 1000;
    if (difference >= 0) {
      return true;
    }
    return Math.abs(System.currentTimeMillis()) < THIRTY_SECONDS;
  }

  public static void generateOidcTokenExpiredExceptionIfTokenNeedFresh(String accesstoken,
          long tokenExpiresTime)
          throws OidcTokenExpiredException {
    // We are in a micro-service configuration don't need to check the expires time
    if (tokenExpiresTime == -1) {
      return;
    }
    if (isTokenNeedFresh(tokenExpiresTime)) {
      throw new OidcTokenExpiredException(
              "Token " + accesstoken + " is expired or will expired in less 30 seconds (Expiration at "
                      + tokenExpiresTime + ")",
              accesstoken, tokenExpiresTime);
    }

  }

  public static void generateOidcTokenExpiredExceptionIfTokenNeedFresh()
          throws OidcTokenExpiredException {
    // We are in a micro-service configuration don't need to check the expires time
    if (UserTool.getUserPrincipal().getAccessTokenExpiresTime() == -1) {
      return;
    }
    if (isTokenNeedFresh(UserTool.getUserPrincipal().getAccessTokenExpiresTime())) {
      throw new OidcTokenExpiredException(
              "Token " + UserTool.getUserPrincipal().getAccessToken()
                      + " is expired or will expired in less 30 seconds (Expiration at " + UserTool
                      .getUserPrincipal().getAccessToken()
                      + ")",
              UserTool.getUserPrincipal().getAccessToken(),
              UserTool.getUserPrincipal().getAccessTokenExpiresTime());
    }

  }

}
