package ch.unige.solidify.test.auth;

import static org.hamcrest.CoreMatchers.is;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.util.Map;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockHttpServletRequestBuilder;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.context.WebApplicationContext;

import ch.unige.solidify.auth.SsoAuthenticationFilter;
import ch.unige.solidify.auth.util.SsoUserData;
import ch.unige.solidify.test.auth.config.MockApplication;
import ch.unige.solidify.test.auth.data.MockData;

/**
 * unit test for SsoAuthenticationFilter
 */

@RunWith(SpringRunner.class)
@SpringBootTest(classes = MockApplication.class)
public class OIDCFilterTest {

  @Autowired
  @InjectMocks
  private SsoAuthenticationFilter ssoAuthenticationFilter;

  @Autowired
  private WebApplicationContext context;

  @Mock
  private RestTemplate restTemplate;

  private MockMvc mockMvc;
  private MockHttpServletRequestBuilder requestBuilder;

  @Before
  public void before() {
    mockMvc = MockMvcBuilders.webAppContextSetup(context).addFilters(ssoAuthenticationFilter).build();
  }

  @Test
  public void testOidcUserWithTokenExpired() throws Exception {
    HttpHeaders oidcHeaders = new HttpHeaders();
    Map<String, Object> oidcHeadersMockData = MockData.getOidcHeadersMockData();
    for (String key : oidcHeadersMockData.keySet()) {
      if (key.equals("OIDC_access_token_expires")) {
        oidcHeaders.set(key, Long.toString((Long) oidcHeadersMockData.get(key)));
      } else {
        oidcHeaders.set(key, (String) oidcHeadersMockData.get(key));
      }
    }
    requestBuilder = MockMvcRequestBuilders.get("/internalInfo").headers(oidcHeaders);
    mockMvc.perform(requestBuilder).andExpect(status().is(498))
            .andExpect(jsonPath("$.tokenExpired", is(oidcHeaders.get("oidc_access_token").get(0))))
            .andExpect(jsonPath("$.tokenTimeExpired",
                    is(Integer.parseInt(oidcHeaders.get("OIDC_access_token_expires").get(0)))));
  }

  // TODO implement mock for GLUU calling in the next technique sprint
  public void testOidcUserAttributsForMicroService() throws Exception {
    HttpHeaders oidcHeaders = new HttpHeaders();
    Map<String, Object> oidcHeadersMockData = MockData.getOidcHeadersMockData();

    for (String key : oidcHeadersMockData.keySet()) {
      if (key.equals("OIDC_access_token_expires")) {
        oidcHeaders.set(key, "-1");
      } else {
        oidcHeaders.set(key, (String) oidcHeadersMockData.get(key));
      }
    }
    SsoUserData user = new SsoUserData();
    user.setFirstname("test");
    user.setOauthToken("xxxxx");
    Mockito.when(restTemplate.postForEntity(Mockito.any(String.class), Mockito.any(HttpEntity.class), Mockito.any(Class.class)))
            .thenReturn(new ResponseEntity<>(user, HttpStatus.OK));

    requestBuilder = MockMvcRequestBuilders.get("/user").headers(oidcHeaders);
    mockMvc.perform(requestBuilder).andExpect(status().is(200)).andDo(print());
  }
}
