package ch.unige.solidify.test.auth;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.doAnswer;

import java.io.IOException;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.mockito.stubbing.Answer;

import com.fasterxml.jackson.databind.ObjectMapper;

import ch.unige.solidify.auth.oidc.OIDCUser;
import ch.unige.solidify.auth.oidc.OidcUserInfoService;
import ch.unige.solidify.model.sso.SsoUser;
import ch.unige.solidify.test.auth.data.MockData;

/**
 * unit test for Creation of a user from HashMap who contains shibboleth headers value Creation of a
 * user from HashMap who contains OID headers value class SsoUserdetails and SsoAuthentication
 */

@RunWith(MockitoJUnitRunner.class)
public class OidcSsoUserTest {

  private OIDCUser oidcUser = new OIDCUser();

  @Mock
  private OidcUserInfoService oidcUserInfoService;

  private Map<String, Object> oidcHeaders = null;

  @Before
  public void setup() {
    oidcHeaders = MockData.getOidcHeadersMockData();
    oidcHeaders.replaceAll((key, oldValue) -> oidcHeaders.get(key));

    oidcUser.setOauthToken((String) MockData.getOidcHeadersMockData().get("oidc_access_token"));
    oidcUser.setUniqueId((String) MockData.getOidcHeadersMockData().get("oidc_claim_sub"));
    doAnswer((Answer<OIDCUser>) invocation -> oidcUser).when(oidcUserInfoService)
            .getOidcUser((String) MockData.getOidcHeadersMockData().get("oidc_access_token"),
                    (long) MockData.getOidcHeadersMockData().get("OIDC_access_token_expires"));
  }

  @Test
  public void testOidcIdUser() {
    OIDCUser user = this.oidcUserInfoService
            .getOidcUser((String) MockData.getOidcHeadersMockData().get("oidc_access_token"),
                    (long) MockData.getOidcHeadersMockData().get("OIDC_access_token_expires"));
    assertTrue(user.toString().length() > 10);
    assertEquals(MockData.OcidcId, user.getUniqueId());
    assertEquals(MockData.oauthAccessToken, user.getAccessToken());
    SsoUser ssoUserData = OIDCUser.toUserData(user);
    assertEquals(MockData.OcidcId, ssoUserData.getUniqueId());
    assertEquals(MockData.oauthAccessToken, ssoUserData.getAccessToken());
  }

  @Test
  public void oidcJacksonString2ArrayTest() throws IOException {
    String jsonAsString = "{\"email\":\"Raoli.Wang@unige.ch\",\"sub\":\"135192@unige.ch\","
            + "\"isMemberOf\":\"webreporting:user\","
            + "\"affiliation\":\"member\","
            + "\"employeeType\":\"EDTUDIANT\","
            + "\"unigeChStudentCategory\":\"EDTUDIANT\","
            + "\"roles\":\"user\""
            + "}";
    ObjectMapper mapper = new ObjectMapper();
    OIDCUser user = mapper.readValue(jsonAsString, OIDCUser.class);
    assertEquals("135192@unige.ch", user.getUniqueId());
    assertEquals(1, user.getIsmemberOf().size());
    assertEquals(1, user.getAffiliation().size());
    assertEquals(1, user.getEmployeeType().size());
    assertEquals(1, user.getUnigeChStudentCategory().size());
    assertEquals(1, user.getRoles().size());
    String jsonAsString2 = "{\"email\":\"Raoli.Wang@unige.ch\",\"sub\":\"135192@unige.ch\","
            + "\"isMemberOf\":[\"webreporting:user\",\"internalinvoicing:user\"],"
            + "\"affiliation\":[\"member\",\"staff\"],"
            + "\"employeeType\":[\"webreporting:user\",\"internalinvoicing:user\"],"
            + "\"unigeChStudentCategory\":[\"ETUDIANT\",\"STAFF\"],"
            + "\"roles\":[\"user\",\"editor\"]"
            + "}";
    user = mapper.readValue(jsonAsString2, OIDCUser.class);
    assertEquals(2, user.getIsmemberOf().size());
    assertEquals(2, user.getAffiliation().size());
    assertEquals(2, user.getEmployeeType().size());
    assertEquals(2, user.getUnigeChStudentCategory().size());
    assertEquals(2, user.getRoles().size());
  }

}
