package ch.unige.solidify.test.auth;

import static org.hamcrest.CoreMatchers.is;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.util.Map;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.HttpHeaders;
import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockHttpServletRequestBuilder;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.util.NestedServletException;

import ch.unige.solidify.auth.SsoAuthenticationFilter;
import ch.unige.solidify.test.auth.config.MockApplication;
import ch.unige.solidify.test.auth.data.MockData;

/**
 * unit test for SsoAuthenticationFilter
 */

@RunWith(SpringRunner.class)
@SpringBootTest(classes = MockApplication.class)
public class ShibboFilterTest {

  @Autowired
  private SsoAuthenticationFilter ssoAuthenticationFilter;

  @Autowired
  private WebApplicationContext context;

  private MockMvc mockMvc;
  private MockHttpServletRequestBuilder requestBuilder;

  @Before
  public void before() {
    mockMvc = MockMvcBuilders.webAppContextSetup(context).addFilters(ssoAuthenticationFilter)
            .build();
  }

  @Test
  public void testEditorRoleAcessAPI() throws Exception {
    HttpHeaders shibboHeaders = new HttpHeaders();
    Map<String, String> shibboHeadersMockData = MockData.getShibboHeadersMockData();
    for (String key : shibboHeadersMockData.keySet()) {
      shibboHeaders.set(key, shibboHeadersMockData.get(key));
    }
    requestBuilder = MockMvcRequestBuilders.get("/api/user").headers(shibboHeaders);
    // mockMvc.perform(requestBuilder).andExpect(MockMvcResultMatchers.status().isOk());
    mockMvc.perform(requestBuilder).andExpect(status().isOk()).andDo(print())
            .andExpect(content().contentType("application/json"))
            .andExpect(jsonPath("$.email", is("test@unige.ch")))
            .andExpect(jsonPath("$.uniqueId", is(MockData.ShibboUniqueId)))
            .andExpect(jsonPath("$.cnindividu", is(MockData.employeeNumber)))
            .andExpect(jsonPath("$.matriculationnumber", is(MockData.matriculationNumber)))
            .andExpect(jsonPath("$.userContext", is("TEC")));
  }

  @Test
  public void testPublicApi() throws Exception {
    HttpHeaders emptyHeader = new HttpHeaders();
    requestBuilder = MockMvcRequestBuilders.get("/api/greet").headers(emptyHeader);
    mockMvc.perform(requestBuilder).andExpect(MockMvcResultMatchers.status().isOk());
  }

  @Test(expected = NestedServletException.class)
  public void testFullyAuthenticatedApi() throws Exception {
    SecurityContextHolder.getContext().setAuthentication(
            new AnonymousAuthenticationToken("VISITOR", "USERNAME",
                    AuthorityUtils.createAuthorityList("ROLE_USER")));
    HttpHeaders emptyHeader = new HttpHeaders();
    requestBuilder = MockMvcRequestBuilders.get("/api/internalInfo").headers(emptyHeader);
    mockMvc.perform(requestBuilder);
  }
}
