package ch.unige.solidify.test.cache;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cache.concurrent.ConcurrentMapCacheManager;
import org.springframework.cache.support.NullValue;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import ch.unige.solidify.auth.oidc.OIDCUser;
import ch.unige.solidify.auth.oidc.OidcUserFactory;
import ch.unige.solidify.auth.util.OidcTokenExpiredException;

@RunWith(SpringRunner.class)
@ContextConfiguration
public class OidcSsoUserCacheTest {

  @Autowired
  CacheManager manager;
  @Autowired
  OidcUserFactory oidcUserFactory;
  OIDCUser user1 = new OIDCUser();
  OIDCUser user2 = new OIDCUser();
  String accessToken = "254888";
  long accessTokenExpired = 1556111841;
  Map<String, String> headers = new HashMap<>();

  @Before
  public void setup() throws OidcTokenExpiredException {
    // Set up the mock to return *different* objects for the first and second call
    Mockito.when(oidcUserFactory
            .get(Mockito.any(String.class), Mockito.any(long.class), Mockito.any(Map.class)))
            .thenReturn(user1, user2);
  }

  @Test
  public void userDetailsInvocationShouldBeCached() throws OidcTokenExpiredException {
    // First invocation returns object returned by the method
    Object result = oidcUserFactory.get(accessToken, accessTokenExpired, headers);
    Assert.assertEquals(result, user1);
    // Second invocation should return cached value, not second (see setup method)
    result = oidcUserFactory.get(accessToken, accessTokenExpired, headers);
    Assert.assertEquals(result, user1);
    // Verify repository method was invoked once
    Mockito.verify(oidcUserFactory, Mockito.times(1)).get(accessToken, accessTokenExpired, headers);
    ArrayList params = new ArrayList();
    params.add(accessToken);
    params.add(accessTokenExpired);

    Cache cache = manager.getCache(OidcUserFactory.OIDC_USER_CACHE_NAME);
    java.util.concurrent.ConcurrentHashMap cach = (java.util.concurrent.ConcurrentHashMap) cache
            .getNativeCache();
    for (Object onekey : cach.keySet()) {
      Object object = cach.get(onekey);
      if (object != null && !(object instanceof NullValue)) {
        Assert.assertTrue(object instanceof OIDCUser);
        ArrayList keyObejct = (ArrayList) onekey;
        Assert.assertEquals(keyObejct.get(0), accessToken);
        Assert.assertEquals(keyObejct.get(1), accessTokenExpired);
      }
    }
    // refresh cache
    oidcUserFactory.reFreshOidcUserCache();
    cach = (java.util.concurrent.ConcurrentHashMap) cache.getNativeCache();
    // second user must be used and returned
    result = oidcUserFactory.get(accessToken, accessTokenExpired, headers);
    Assert.assertEquals(result, user2);
  }

  @Configuration
  @EnableCaching
  static class Config {

    // Simulating ODIC caching configuration
    @Bean
    CacheManager cacheManager() {
      return new ConcurrentMapCacheManager(OidcUserFactory.OIDC_USER_CACHE_NAME);
    }

    @Bean
    OidcUserFactory oidcUserFactory() {
      return Mockito.mock(OidcUserFactory.class);
    }
  }

}
