package ch.unige.solidify.controller;


import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.rest.SubResourceContainer;
import ch.unige.solidify.service.CompositeResourceService;

public abstract class CompositionController<T extends Resource<T> & SubResourceContainer, V extends Resource<V>>
        extends SubResourceController<T, V> {

  @Autowired
  protected CompositeResourceService<V> childItemService;

  @PostMapping
  public HttpEntity<V> create(@PathVariable final String parentid,
          final @Valid @RequestBody V childItem) {

    if (childItem.getResId() != null && this.childItemService.existsById(childItem.getResId())) {
      throw new SolidifyRuntimeException(this.messageService.get("validation.resource.alreadyexist",
              new Object[] { childItem.getResId() }));
    }

    T parentItem = this.itemService.findOne(parentid);

    this.setAuthenticatedUserProperties(childItem);

    if (parentItem.addItem(childItem)) {

      this.setAuthenticatedUserProperties(parentItem);

      this.itemService.save(parentItem);
      return new ResponseEntity<>(childItem, HttpStatus.CREATED);
    }
    return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
  }

  @RequestMapping(value = SolidifyConstants.URL_ID, method = { RequestMethod.PUT,
          RequestMethod.PATCH })
  public HttpEntity<V> update(@PathVariable final String parentid, @PathVariable final String id,
          @RequestBody final V newChildItem) {

    T oldParentItem = this.itemService.findOne(parentid);

    V oldChildItem = this.itemService.findByParentIdAndChildId(parentid, getMainResourceClass(),
            getSubResourceClass(), id);
    if (oldChildItem == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    this.setAuthenticatedUserProperties(newChildItem);

    if (oldChildItem.update(newChildItem)) {
      this.setAuthenticatedUserProperties(oldParentItem);
      this.itemService.save(oldParentItem);
    }
    return new ResponseEntity<>(oldChildItem, HttpStatus.OK);
  }

  @DeleteMapping(value = SolidifyConstants.URL_ID)
  public HttpStatus delete(@PathVariable final String parentid, @PathVariable final String id) {
    T oldParentItem = this.itemService.findOne(parentid);

    V childItem = this.childItemService.findOne(id);
    if (oldParentItem.removeItem(childItem)) {
      this.itemService.save(oldParentItem);
      return HttpStatus.OK;
    }
    return HttpStatus.GONE;
  }

  @Override
  protected void addLinks(final String parentId, final V childItem) {
    childItem.removeLinks();
    childItem.addLinks(linkTo(methodOn(this.getClass()).list(parentId, null, null)), false, true);
  }
}
