package ch.unige.solidify.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.repository.SolidifyRepositoryNoSql;
import ch.unige.solidify.rest.ResourceNoSql;

public abstract class ResourceNoSqlController<T extends ResourceNoSql<T>>
        extends ResourceNoSqlReadOnlyController<T> implements ControllerWithHateoasHome {

  @Autowired
  public ResourceNoSqlController(SolidifyRepositoryNoSql<T> itemRepository) {
    super(itemRepository);
  }

  @PostMapping
  public HttpEntity<T> create(@RequestBody T t) {
    T item = itemRepository.save(t);
    return new ResponseEntity<>(item, HttpStatus.CREATED);
  }

  @RequestMapping(value = SolidifyConstants.URL_ID, method = { RequestMethod.PUT,
          RequestMethod.PATCH })
  public HttpEntity<T> update(@PathVariable String id, @RequestBody T t2) {
    T t1 = itemRepository.findOne(this.getIndex(), id);
    if (t1 == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    if (t1.update(t2)) {
      T item = itemRepository.update(t2);
      return new ResponseEntity<>(item, HttpStatus.OK);
    }
    return new ResponseEntity<>(t1, HttpStatus.OK);
  }

  @DeleteMapping(value = SolidifyConstants.URL_ID)
  public HttpStatus delete(@PathVariable String id) {
    T t = itemRepository.findOne(this.getIndex(), id);
    if (t == null) {
      return HttpStatus.NOT_FOUND;
    }
    if (!itemRepository.delete(t)) {
      return HttpStatus.GONE;
    }
    return HttpStatus.OK;
  }

}