package ch.unige.solidify.util;

import java.io.IOException;
import java.io.InputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.model.ChecksumAlgorithm;

public class ChecksumTool {

  private ChecksumTool() {
    throw new IllegalStateException(SolidifyConstants.TOOL_CLASS);
  }

  // *************************
  // ** Checksums functions **
  // *************************

  public static String[] computeChecksum(InputStream inputStream,
          List<ChecksumAlgorithm> algorithms) throws NoSuchAlgorithmException, IOException {
    String[] algos = new String[algorithms.size()];
    for (int i = 0; i < algorithms.size(); i++) {
      algos[i] = algorithms.get(i).name();
    }

    return ChecksumTool.computeChecksum(inputStream, algos);
  }

  public static String[] computeChecksum(InputStream inputStream, String[] checksums)
          throws NoSuchAlgorithmException, IOException {
    MessageDigest[] digests = new MessageDigest[checksums.length];
    for (int i = 0; i < checksums.length; i++) {
      digests[i] = getMessageDigest(ChecksumAlgorithm.valueOf(checksums[i]));
    }
    return getFileChecksum(inputStream, digests);
  }

  private static MessageDigest getMessageDigest(ChecksumAlgorithm checksumAlgo)
          throws NoSuchAlgorithmException {

    return MessageDigest.getInstance(checksumAlgo.toString());
  }

  private static String[] getFileChecksum(InputStream inputStream, MessageDigest[] digests)
          throws IOException {
    String[] checksums = new String[digests.length];
    byte[] byteArray = new byte[1024];
    int bytesCount;

    // Read file data and update in message digest
    while ((bytesCount = inputStream.read(byteArray)) != -1) {
      for (MessageDigest digest : digests) {
        digest.update(byteArray, 0, bytesCount);
      }
    }
    inputStream.close();

    for (int i = 0; i < digests.length; i++) {
      checksums[i] = StringTool.toHexaString(digests[i].digest());
    }

    return checksums;
  }

  public static List<MessageDigest> getMessageDigests(
          List<ch.unige.solidify.model.ChecksumAlgorithm> algorithms)
          throws NoSuchAlgorithmException {

    List<MessageDigest> digests = new ArrayList<>();

    for (int i = 0; i < algorithms.size(); i++) {
      digests.add(MessageDigest.getInstance((algorithms.get(i).toString())));
    }

    return digests;
  }

  public static List<String> getChecksums(InputStream stream,
          List<ChecksumAlgorithm> algorithmsToCalculate)
          throws NoSuchAlgorithmException, IOException {

    /*
     * Get all MessageDigest
     */
    List<MessageDigest> digests = ChecksumTool.getMessageDigests(algorithmsToCalculate);

    /*
     * Read stream and calculate checksums
     */
    byte[] byteArray = new byte[1024];
    int bytesCount;
    while ((bytesCount = stream.read(byteArray)) != -1) {
      for (MessageDigest digest : digests) {
        digest.update(byteArray, 0, bytesCount);
      }
    }
    stream.close();

    /*
     * For each MessageDigest, get the hash value and fill the result
     */
    List<String> checksums = new ArrayList<>();
    for (int i = 0; i < digests.size(); i++) {
      checksums.add(StringTool.toHexaString(digests.get(i).digest()));
    }

    return checksums;
  }
}
